/*
 * Copyright 1995 untill 2011 RMIB (see the readme for contact information)
 *
 * This file is part of libxrit
 *
 * libxrit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libxrit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libxrit.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RMIBSMALLLIB_H
#define RMIBSMALLLIB_H

/*  A LOT OF SYSTEM HEADERS (TRY TO COMMENT THEM) */

#include <sys/ipc.h>
#include <sys/mman.h>
#include <sys/resource.h>
#include <sys/sem.h>
#include <sys/shm.h>
#include <sys/stat.h>
#include <sys/statvfs.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/utsname.h>

#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <inttypes.h>
#include <limits.h>
#include <malloc.h>
#include <math.h>
#include <regex.h>
#include <setjmp.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <float.h>

#ifdef __DECC
#ifdef _XOPEN_SOURCE_EXTENDED
#undef  _XOPEN_SOURCE_EXTENDED
#endif
#define _XOPEN_SOURCE_EXTENDED
#include <ftw.h>
#else
#ifdef __USE_XOPEN_EXTENDED
#undef  __USE_XOPEN_EXTENDED
#endif
#define __USE_XOPEN_EXTENDED
#include <ftw.h>
#endif

#include <zlib.h>
#include <bzlib.h>

#define NOX
#ifndef NOX
#include <X11/X.h>
#include <X11/Xatom.h>
#include <X11/Xlib.h>
#include <X11/Xlibint.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#endif

#define NOMAGICK
#ifndef NOMAGICK
#include <magick/api.h>
#endif

#define NOHDF
#ifndef NOHDF
#include <hdf5.h>
#endif

/* extern variables */

/* LG
	extern struct _Err {
		int ErrStat;
		int ErrActive;
		int ErrCode;
	} Err;
*/
extern int ErrCount;
extern int WarnCount;



extern int bats_display_byte_swap;

/* ------------------------------------------------------------------------ */
/* If these macros have already been defined elsewhere by any other header, */
/*  we should first undefine them to avoid warnings.                        */

#undef PYTHAG
#undef MIN
#undef MAX
#undef SQR
#undef SIGN
#undef sign
#undef DSIGN

#define MIN(a,b) (((a) < (b)) ? (a) : (b))

/* avoiding system specific functions: not optimal but it works...*/
#define i386Round ROUND



/*  BATS LIBRARY DEFINES */

#define ALLOCATION_ERR "allocation error"

#define bit          unsigned char

#define UNLIKELY(Condition) (!!(Condition))

#define ERROR_CLEAR _ErrClear()

#define CLAMP(a, min, max) ((a) <= (min) ? (min) : ((a) >= (max) ? (max) : (a)))

#define DIR_COMPLETE 2

#define DOUBLE   12

#define _ERROR_C(ErrorCode) \
{ \
	_OnErr(__LINE__, ErrorCode); \
	goto LabelOfErrorHandling; \
}

#define CCOE_M1(Message, MArg1) \
{ \
    if UNLIKELY(Err.ErrStat) { \
        ErrorPrintf(Message, MArg1); \
        ERROR_CLEAR; \
        continue; \
    } \
}

#define ERROR_END \
		Err.ErrStat = ErrStatLocal; \
		Err.ErrCode = ErrCodeLocal; \
	}

#define ERROR_FILE_LEN  256

#define ERROR_HANDLER_BEGIN(function_name) \
LabelOfErrorHandling: \
	{ \
		int ErrStatLocal = 0, ErrCodeLocal = 0; \
		if UNLIKELY(Err.ErrActive) { \
			ErrStatLocal = Err.ErrStat; \
			ErrCodeLocal = Err.ErrCode; \
			_HandleErr(__FILE__, function_name);

#define ERROR_HANDLER_END \
		}

#define ERROR_HANDLER(function_name) \
	ERROR_HANDLER_BEGIN(function_name); \
	ERROR_HANDLER_END;

#define ERROR_M(Message) \
{ \
	_OnMsg(Message);\
	_ERROR_C(0); \
}

#define ERROR_M1(Message, MArg1) \
{ \
	_OnMsg(Message, MArg1); \
	_ERROR_C(0); \
}

#define ERROR_M2(Message, MArg1, MArg2) \
{ \
	_OnMsg(Message, MArg1, MArg2); \
	_ERROR_C(0); \
}

#define ERROR_M3(Message, MArg1, MArg2, MArg3) \
{ \
	_OnMsg(Message, MArg1, MArg2, MArg3); \
	_ERROR_C(0); \
}

#define ERROR() _ERROR_C(0)

#define ERROR_IF_M1(Condition, Message, MArg1) \
{ \
	if UNLIKELY(Condition) \
		ERROR_M1(Message, MArg1); \
}

#define ERROR_MSG_LEN   256

#define FLOAT    11

#define NUM_LINE_SEGMENT                 464

#define INIT_IP_RECTANGLE(rec) {rec.ul.x=-1;rec.ul.y=-1;rec.lr.x=-1;rec.lr.y=-1;}

#define int2          int16_t

#define INT2     6

#define int4          int32_t

#define INT4     8

#define IP_ImageWrite    BATSIFY(IP_ImageWrite)  

#define JOE \
{ \
	if UNLIKELY(Err.ErrStat) { \
		_OnJOE(__LINE__); \
		goto LabelOfErrorHandling; \
	} \
}

#define LU_LITTLE_ENDIAN   1234

#define INT32_MAX (2147483647)

#define MAX_UINT(type)      (~((type) 0) >> 1)

#define HRIT_SEGMENT_HEADER_LENGTH      (163+3+NUM_LINE_SEGMENT*13)

#define PARAMETER_ERR "incorrect parameter"

#define PROPAGATION_ERR "propagation error"

#define RETURN(ReturnValue) \
	ERROR_END; \
	return ReturnValue;

#define ROUND(a) ((a) < 0. ? (int4) ((a) - .5) : (int4) ((a) + .5))

#define TERMINATE \
	goto LabelOfErrorHandling;

#define TERMINATE_IF(Condition) \
{ \
	if (Condition) \
		TERMINATE; \
}

#define u_c          unsigned char

#define u_int1        uint8_t

#define u_int2        uint16_t

#define U_INT2   5

#define u_int4        uint32_t

#define U_INT4   7

#define u_int_8       uint64_t

#define U_INT_8  9

#define CCOE \
{ \
    if UNLIKELY(Err.ErrStat) { \
        ERROR_CLEAR; \
        continue; \
    } \
}

#define LU_BIG_ENDIAN      4321

#define ERROR_M_C(Message, ErrorCode) \
{ \
	_OnMsg(Message); \
	_ERROR_C(ErrorCode); \
}

#define TYPE_COMPOSITE 128

#define BIT      0

#define U_CHAR   1

#define FLOATING_TYPE(type) (((type) 0.25) && ((type) 0.25 - 1))

#define FASTROUND i386Round

#define _CONVERT_DISCRETE_ERR(dtype, stype) { \
	dtype * _dst = (dtype *) dst; \
	dtype * _end = (dtype *) end; \
	stype * _src = (stype *) src; \
	if (FLOATING_TYPE(stype) && !FLOATING_TYPE(dtype)) { \
		while (_dst < _end) \
        { \
            if (*_src == (stype) serr) \
                *_dst++ = (dtype) derr; \
            else \
                *_dst++ = (dtype) FASTROUND((*_src - offset)/quanta); \
            ++_src; \
        } \
		return; \
	} \
	while (_dst < _end) \
    { \
        if (*_src == (stype) serr) \
            *_dst++ = (dtype) derr; \
        else \
            *_dst++ = (dtype) *_src; \
        _src++; \
    } \
	return; \
}

#define DIRDIR_FLAG 0x80000000U

#define ERROR_IF_V(Condition) \
{ \
	if UNLIKELY(Condition) \
		ERROR(); \
}

#define ERROR_IF_M(Condition, Message) \
{ \
	if UNLIKELY(Condition) \
		ERROR_M(Message); \
}

#define ERROR_IF_M2(Condition, Message, MArg1, MArg2) \
{ \
	if UNLIKELY(Condition) \
		ERROR_M2(Message, MArg1, MArg2); \
}

#define ERROR_IF_M3(Condition, Message, MArg1, MArg2, MArg3) \
{ \
	if UNLIKELY(Condition) \
		ERROR_M3(Message, MArg1, MArg2, MArg3); \
}

#define MAX_FLOAT  FLT_MAX

#define UNSIGNED_TYPE(type) ((type) -1 > (type) 0)

#define _GET_VALUE(stype, idx) { \
	return (double) ((stype *) src)[idx]; \
}

#define SEC_PER_DAY     86400.

#define DIR_PHYS     8

#define DIR_MOUNT    16

#define DIR_FILES    4

#define DIR_SORTED   1

#define REGEX_ICASE 1

#define REGEX_NLINE 2

#define MAX_INT4   INT32_MAX

#define PI M_PI

#define MIN_FLOAT  (-MAX_FLOAT)

#define CHAR     2

#define U_INT1   3

#define INT1     4

#define s_c          signed char

#define int1          int8_t

#define INT_8    10

#define int_8         int64_t

#define float4       float

#define TYPE_ERR "improper type"

#define _SWITCH2(dtype, stype, foo, bar) \
	switch (stype) { \
		case BIT:     foo(dtype, bit, bar) \
		case U_CHAR:  foo(dtype, u_c, bar) \
		case CHAR:    foo(dtype, char, bar) \
		case U_INT1:  foo(dtype, u_int1, bar) \
		case INT1:    foo(dtype, int1, bar) \
		case U_INT2:  foo(dtype, u_int2, bar) \
		case INT2:    foo(dtype, int2, bar) \
		case U_INT4:  foo(dtype, u_int4, bar) \
		case INT4:    foo(dtype, int4, bar) \
		case U_INT_8: foo(dtype, u_int_8, bar) \
		case INT_8:   foo(dtype, int_8, bar) \
		case FLOAT:   foo(dtype, float4, bar) \
		case DOUBLE:  foo(dtype, double, bar) \
		default:      ERROR_M(TYPE_ERR) \
	}

#define P2VOID    14

#define NVZ 9					/* Number of view zenith angles in binary data */

#define ERROR_IF_M_C(Condition, Message, ErrorCode) \
{ \
	if UNLIKELY(Condition) \
		ERROR_M_C(Message, ErrorCode); \
}

#define RGBA        (TYPE_COMPOSITE + 9)

#define _SWITCH1(dtype, stype, foo) \
	switch (dtype) { \
		case BIT:     foo(bit, stype) \
		case U_CHAR:  foo(u_c, stype) \
		case CHAR:    foo(char, stype) \
		case U_INT1:  foo(u_int1, stype) \
		case INT1:    foo(int1, stype) \
		case U_INT2:  foo(u_int2, stype) \
		case INT2:    foo(int2, stype) \
		case U_INT4:  foo(u_int4, stype) \
		case INT4:    foo(int4, stype) \
		case U_INT_8: foo(u_int_8, stype) \
		case INT_8:   foo(int_8, stype) \
		case FLOAT:   foo(float4, stype) \
		case DOUBLE:  foo(double, stype) \
		default:      ERROR_M(TYPE_ERR) \
	}

#define _CONVERT(dtype, stype) { \
	dtype * _dst = (dtype *) dst; \
	dtype * _end = (dtype *) end; \
	stype * _src = (stype *) src; \
	if (FLOATING_TYPE(stype) && !FLOATING_TYPE(dtype)) { \
		while (_dst < _end) { *_dst++ = (dtype) FASTROUND(*_src); ++_src; } \
		return; \
	} \
	while (_dst < _end) { *_dst++ = (dtype) *_src++; } \
	return; \
}

#define _CONVERT_DISCRETE(dtype, stype) { \
	dtype * _dst = (dtype *) dst; \
	dtype * _end = (dtype *) end; \
	stype * _src = (stype *) src; \
	if (FLOATING_TYPE(stype) && !FLOATING_TYPE(dtype)) { \
		while (_dst < _end) \
        { \
            *_dst++ = (dtype) FASTROUND((*_src - offset)/quanta); \
            ++_src; \
        } \
		return; \
	} \
	while (_dst < _end) \
    { \
        *_dst++ = (dtype) *_src; \
        _src++; \
    } \
	return; \
}

#define RANGE_ERR "value outside range"

#define _MIN_I_(type) (UNSIGNED_TYPE(type) ? (type) 0 : \
						(sizeof(type) == 8 ? (type) ((int_8) 1 << (8 * sizeof(type) - 1)) : \
						(type) (1L << (8 * sizeof(type) - 1))))

#define ADRFREE(ptr) { deallocate(ptr); ptr = NULL; }

#define BZ2_BUFFER_SIZE 5000

#define WRITE_ERR "write error"

#define TWO_PI  (2.*PI)

#define _OUT_F_(type, v) \
	ERROR_IF_M2(sizeof(type) == 4 && (v < MIN_FLOAT || v > MAX_FLOAT), \
				"%g: %s of "#type, v, RANGE_ERR)

#define _MAX_I_(type) (UNSIGNED_TYPE(type) ? (type) -1 : \
						(sizeof(type) == 8 ? -((type) ((int_8) 1 << (8 * sizeof(type) - 1)) + 1) : \
						-((type) (1L << (8 * sizeof(type) - 1)) + 1)))

#define _OUT_I_TEST(rc,type,v)  (rc=(((v + .5) <= _MIN_I_(type)) || ((v - .5) >= _MAX_I_(type))))        

#define _OUT_I_(type, v) \
	{int _rc ; ERROR_IF_M2(_OUT_I_TEST(_rc,type,v), \
				"%g: %s of "#type, v, RANGE_ERR)}

#define STRING      (TYPE_COMPOSITE + 0)

#define SM_STRINGS  (TYPE_COMPOSITE + 1)

#define IP_IMAGE    (TYPE_COMPOSITE + 2)

#define IP_IMAGES   (TYPE_COMPOSITE + 3)

#define MP_VECTOR   (TYPE_COMPOSITE + 4)

#define MP_VECTORS  (TYPE_COMPOSITE + 5)

#define MP_MATRIX   (TYPE_COMPOSITE + 6)

#define MP_MATRICES (TYPE_COMPOSITE + 7)

#define MP_INDEX    (TYPE_COMPOSITE + 8)

#define _FILL_VALUE(dtype, w) { \
	int4 _i, _j; \
	if (FLOATING_TYPE(dtype)) { \
		_OUT_F_(dtype, val); \
		for (_j = ymin; _j < ymax; ++_j) \
			for (_i = xmin; _i < xmax; ++_i) \
				((dtype *) dst)[_j * w + _i] = (dtype) val; \
		return; \
	} \
	_OUT_I_(dtype, val); \
	{ \
		dtype _v = (dtype) FASTROUND(val); \
		for (_j = ymin; _j < ymax; ++_j) \
			for (_i = xmin; _i < xmax; ++_i) \
				((dtype *) dst)[_j * w + _i] = _v; \
	} \
	return; \
}

#define _GET_INTVALUE(stype, idx) { \
	stype _s = ((stype *) src)[idx]; \
	if (FLOATING_TYPE(stype)) { \
		_OUT_I_(int4, _s); \
		return (int4) FASTROUND(_s); \
	} \
	_OUT_I_(int4, (double) _s); \
	return (int4) _s; \
}

#define CL_NO_INIT_MESSAGE "CL_Init function must be called before any use of the CL functions"

#define _PUT_VALUE(dtype, idx) { \
	if (FLOATING_TYPE(dtype)) { \
		_OUT_F_(dtype, val); \
		((dtype *) dst)[idx] = (dtype) val; \
		return; \
	} \
	_OUT_I_(dtype, val); \
	((dtype *) dst)[idx] = (dtype) FASTROUND(val); \
	return; \
}

#ifdef __cplusplus
extern "C" {
#endif

/*  BATS LIBRARY TYPEDEF */

typedef struct {
    u_c **data;  /**< A pointer to a set of pointers (or an array of pointers).
                      data[0] till data[height-1] are the pointers to the start of each line. */
                 /* u_c is used as a dummy */
    char *name;  /**< The name of the image. */
    int4 type;   /**< The type of the image (BIT, U_CHAR, CHAR, U_INT1, INT1, U_INT2, INT2, U_INT4, INT4, U_INT_8, INT_8, FLOAT, DOUBLE, LDOUBLE) */
    int4 width;  /**< The with of the image in pixels. */
    int4 height; /**< The height of the image in pixels. */
} IP_image;

	typedef struct {
		IP_image **image;		/* List of images */
		char *name;				/* optional name field */
		int4 num;				/* Number of images */
	} IP_images;

	typedef struct {
		int4 x;					/* x-coordinate of the point */
		int4 y;					/* y-coordinate of the point */
	} IP_point;

	typedef struct {
		IP_point ul;			/* upper left point */
		IP_point lr;			/* lower right point */
	} IP_rectangle;

	typedef struct {
		IP_rectangle *rectangle;
		int4 num;
	} IP_rectangles;

	typedef struct {
		char **str;				/* Table of strings */
		char *name;				/* Name */
		int4 num;				/* Number of strings */
	} SM_strings;

	typedef struct {
		double *coef;
		char *name;
		int4 type;
		int4 dim;
	} MP_vector;

	typedef struct {
		int4 *ind;
		char *name;
		int4 dim;
	} MP_index;

	typedef struct {
		MP_vector **prob;
		MP_index *cclass;
		char **name;
		char *global_name;

		int4 nb_elements;		/* number total of elements in the DATABASE */
		int4 nb_classes;
	} CP_classif;

	typedef struct {
		MP_vector **data;		/* x[i,j] i -> nb elements, j -> nb_classes */
		CP_classif *theo;
		CP_classif *estm;
		char *name;				/* database name if exists */

		int4 nb_elements;		/* number total of elements in the DATABASE */
		int4 x_dim;
		int4 nb_iterations;		/* do not use */
	} CP_dataset;

/*  BATS LIBRARY STRUCT */

	extern struct _Err {
		int ErrStat;
		int ErrActive;
		int ErrCode;
	} Err;

/*  BATS LIBRARY ENUM */

/*  BATS LIBRARY GLOBAL VARIABLE */

/*  BATS LIBRARY FUNCTION PROTOTYPES */

 void *allocate(size_t size);
 int4 ByteSex(void);
 void deallocate(void *ptr);
 int4 CL_InitFlag(void);
 int4 SizeOf(int4 type);
 void convert_discrete(void *dst, int4 dtype, const void *src, int4 stype, size_t nelem, double quanta, double offset);
 void convert_discrete_error(void *dst, int4 dtype, const void *src, int4 stype, size_t nelem, double quanta, double offset, double serr, double derr);
 void SM_StringsFree(SM_strings * s);
 char *StringCopy(const char *str);
 void _ErrClear(void);
 void fillvalue(void *dst, int4 type, int4 width, int4 xmin, int4 xmax, int4 ymin, int4 ymax, double val);
 FILE *FM_FileOpen(const char *path, const char *mode);
 int4 _FM_FileExistence(const char *file_path, struct stat *p_buf);
 int4 getintvalue(void *src, int4 type, size_t idx);
 double getvalue(void *src, int4 type, size_t idx);
 void _HandleErr(const char *file_name, const char *function_name);
 void _HandleWarn(const char *file_name, int line_no);
 double IP_DistanceBetweenPoints(IP_point point1, IP_point point2);
 void IP_ImageFree(IP_image * image);
 IP_image *IP_ImageAlloc(int4 type, int4 width, int4 height);
 void convert(void *dst, int4 dtype, const void *src, int4 stype, size_t nelem);
 IP_image *IP_ImageConvertTypeDiscrete(IP_image * im, int4 dest_type, double quanta, double offset);
 IP_image *IP_ImageConvertTypeDiscreteError(IP_image * im, int4 dest_type, double quanta, double offset, double serr, double derr);
 IP_image *IP_ImageCopy(IP_image * image);
 int4 IP_ImageRectangleIntersection(IP_image * image, IP_rectangle * rectangle, int4 * xmin, int4 * xmax, int4 * ymin, int4 * ymax);
 IP_image *IP_ImageExtractRectangle(IP_image * sourceImage, IP_rectangle * rectangle);
 void IP_ImageFillRectangle(IP_image * im, IP_rectangle * rec, double value);
 void IP_ImageAdrFree(IP_image ** image);
 int4 IP_ImageGetIntValue(IP_image * im, int4 x, int4 y);
 double IP_ImageGetValue(IP_image * im, int4 x, int4 y);
 void putvalue(void *dst, int4 type, size_t idx, double val);
 void ErrorPrintf(const char *format, ...);
 IP_image *IP_ImageExtract(IP_image * sourceImage, IP_rectangle * rectangle);
 void *reallocate(void *ptr, size_t size);
 void IP_ImagesFree(IP_images * images);
 IP_images *IP_ImagesAlloc(int4 num);
 IP_images *IP_ImagesCopy(IP_images * ims);
 IP_images *IP_ImagesExtract(IP_images * ims, int4 offset, int4 sampling_ratio, int4 num);
 void IP_ImagesAdrFree(IP_images ** images);
 int IP_IsDefaultRectangle(IP_rectangle rec);
 int4 IP_PointPositionInSegment(IP_point p, IP_point p1, IP_point p2);
 void MP_CartesianToPolar(double x, double y, double *radius, double *angle);
 void IP_RectangleDisplay(IP_rectangle * rec);
 IP_point IP_RectangleShortestPath(IP_rectangle Rect1, IP_rectangle Rect2);
 int4 IP_RectangleSurroundPoint(IP_point p1, IP_rectangle rec);
 int IP_RectangleHasBiggerSide(IP_rectangle leftRec, IP_rectangle rightRec);
 int4 IP_RectangleInclude(IP_rectangle rec1, IP_rectangle rec2);
 IP_rectangle IP_RectangleIntersection(IP_rectangle rec1, IP_rectangle rec2);
 int IP_RectangleOverlap(IP_rectangle rec1, IP_rectangle rec2);
 IP_rectangles *IP_RectanglesAlloc(int4 num);
 double IP_RectangleDistanceFromPoint(IP_point point, IP_rectangle rec);
 void IP_RectanglesFree(IP_rectangles * recs);
 double IP_RectangleDistance(IP_rectangle Rect1, IP_rectangle Rect2);
 void IP_RectanglesReallocate(IP_rectangles * recs, int4 num);
 int4 IP_RectangleSurface(IP_rectangle rec);
 int IP_RectangleSurroundCoordinate(IP_rectangle rec, int4 x, int4 y);
 IP_rectangle IP_RectangleUnion(IP_rectangle Rect1, IP_rectangle Rect2);
 void IP_RectangleTranslate(IP_rectangle * rec, IP_point pt);
 void IP_RectanglesCombine(IP_rectangles * recs, double maximal_distance);
 double LU_CdsTimeToUnix(u_int2 day, u_int4 millisec );
 int4 FM_FileExistence(const char *file_path);
 void LU_UnixTimeToDate(double U_time, int4 * year, int4 * month, int4 * day, int4 * hour, int4 * minute, double *second, int4 * yday, int4 * wday);
 SM_strings *SM_StringsAlloc(int4 num);
 void SM_StringsReallocate(SM_strings * s, int4 num);
 char *SM_StringExtract(char *str, int4 deb, int4 end);
 char *SM_StringExtractBasenameLevel(char *str, int4 level);
 int4 _CL_StringSearch(char *str, SM_strings * comline);
 void MP_PolarToCartesian(double radius, double angle, double *x, double *y);
 char *StringType(int4 type);
 IP_point IP_PointRotate(IP_point src, double alpha, IP_point ctr);
 void _OnErr(int line_no, int err_code);
 void _OnJOE(int line_no);
 void _OnMsg(const char *Message, ...);
 int4 IP_ImagePutValue(IP_image * im, int4 x, int4 y, double value);
 int4 IP_ImagePutValueIfPossible(IP_image * im, int4 x, int4 y, double value);
 void IP_ImagesAddImage(IP_images * ims, IP_image * im);
 void SetErrorFile(const char *Name);
 void SetVerbosity(int verb);
 IP_image *IP_ImageConvertType(IP_image * im, int4 dest_type);
 int4 SM_StringIsEmpty(const char *str);
 SM_strings *SM_StringsAddString(SM_strings * strings, char *str);
 SM_strings *SM_StringsAddStringPointer(SM_strings * strings, char *str);
 void SM_StringsAddStrings(SM_strings * s1, SM_strings * s2);
 SM_strings *SM_StringsCopy(SM_strings * s);
 void SM_StringsAdrFree(SM_strings ** s);
 SM_strings *SM_StringsAddToStrings(SM_strings * strs, const char *list, ...);
 void SM_StringReplaceStringNOccurences(char *str, char *str_old, char *str_new, int4 n_occur);
 SM_strings *SM_StringsAllocList(const char *list, ...);
 SM_strings *SM_StringsAddStrings2(SM_strings * s1, SM_strings * s2);
 void SM_StringsDisplay(SM_strings * s);
 void SM_StringsDisplayErr(SM_strings * s);
 char *LU_GetLine(FILE * stream);
 SM_strings *LU_ReadStream(FILE * stream);
 void SM_StringsReorganise(SM_strings * s);
 void SM_StringsRemoveEmptyStrings(SM_strings * s);
 void SM_StringsWrite(const char *filename, const SM_strings * s);
 char *StringConcat(const char *str1, const char *str2);
 void SM_StringReplaceString(char *str, char *str_old, char *str_new);
 void CL_Free(void);
 int4 FM_FileRemove(char *name);
 int4 SM_StringHasSuffix(char *str, char *suf);
 char *SM_StringExtractBasename(char *str);
 MP_index *MP_IndexAlloc(int4 dim);
 int4 FM_FileClose(FILE * fp);
 off_t FM_FileSize(const char *name);
 void LU_DisplayTime(double U_time);
 SM_strings *SM_StringsRead(const char *filename);
 void LU_StringsRegexFilter(SM_strings * list, const char *filter, int4 flags);
 void SM_StringsSort(SM_strings * s);
 void MP_IndexFree(MP_index * indx);
 char *LU_StringExtractRegex(char* str, const char *filter, int4 flags);
 int4 CL_StringSearch(char *str);
 void byte_swap_16(void * data, size_t num);
 char *CL_StringProcessing(char *str);
 char *StringsConcat(int4 nb, ...);
 SM_strings *LU_ScanDirectory(const char *directory, const char *regex, int4 depth, int4 mode);
 int4 FM_FileDataSyncAndClose(FILE * fp);
 int4 FM_FileBunzip2(char *comp_name, char *uncomp_name);
 int4 FM_FileUncompress(char *gz_name, char *guz_name);
 char *LU_DataPreProcess(char *in, int4 *flag);
 int4 CL_Opt(char let);
 void byte_swap_32(void * data, size_t num);
 void byte_swap_64(void * data, size_t num);
 void byte_swap(void *data, size_t num, int4 data_type);
 void MP_DataByteSwap(void *val, int4 type);
 void CL_Init(int4 argc, char **argv);
 int4 LU_Verbose(char *format, ...);

#define MIN_INT4   INT32_MIN

SM_strings *CL_GetStrings(int4 start_pos);
int4 SM_StringExtractInt(char *str, int4 deb, int4 end);
int4 ShortJulianDayFromMonthAndDay(int4 year, int4 m, int4 d);
int4 CL_Sopt(char *str);
char *CL_GetStrOpt(char let);
int4 SM_StringIsIntegerValue(char *str);
char *CL_GetStrSopt(char *str);
int4 SM_StringToInt(char *str);
int4 LU_YearIsLeap(int4 year);

#ifdef __cplusplus
}
#endif

#endif /* RMIBSMALLIBL_H */
