/*****************************************************************************80
****  DADF MODULE HEADER   ***

TYPE:
Main program.

PURPOSE:
Command line tool for manual decompression of HRIT/LRIT files.

FUNCTION:
Command line tool for manual decompression of HRIT/LRIT files.
Used for integration testing and in the off-line environment.

INTERFACES:
See Usage() function.

RESOURCES:
Heap Memory (>2K).

REFERENCES:
None.

PROCESSING:
Opens the specified compressed HRIT/LRIT file, decompresses it and writes
the decompressed file to the current working directory.

DATA:
See 'DATA:' in the class header below.

LOGIC:
-

****  END MODULE HEADER   ***
*******************************************************************************/

#include <stdio.h>

#include "ErrorHandling.h"			// Util
#include "CxRITFile.h"				// DISE
#include "CxRITFileDecompressed.h"	// DISE




// Description:	Tells the correct command line syntax.
// Returns:		Nothing.
int Usage
(
	const std::string& i_ProgramName
)
{
	std::cout << "Usage: " << i_ProgramName	<< " <Name of compressed HRIT/LRIT file>\n";
	return 1;
}




// Description:	Entry function of program.
// Returns:		0 on success,
//				1 on error.
int main
(
	int   argc,
	char* argv[]
)
{
	// Check completeness of command line parameters.
	if (argc < 2)
	{
		return Usage(argv[0]);
	}

	// Open input file.
	DISE::CxRITFile compressedFile;
	try
	{
		compressedFile = DISE::CxRITFile(argv[1]);
	}
	catch (...)
	{
		std::cout << "Opening/reading input file failed.\n";
		return Usage(argv[0]);
	}

	// Decompress input file.
	DISE::CxRITFileDecompressed decompressedFile(compressedFile);

	// Store decompressed file.
//	decompressedFile.Write(decompressedFile.GetAnnotation().GetText());
//	std::cout << "Decompressed file: " << decompressedFile.GetAnnotation().GetText() << "\n";

	return 0;
}
